﻿:Class PrimeServerSecure : #.Conga.Connection

    ∇ cert←ReadTestCert name;fn;ldrc;certpath
   ⍝ Return an X509 object based on files in the TestCertificates folder (assumes existence)
     
      certpath←(2 ⎕NQ'.' 'GetEnvironment' 'Dyalog'),'/TestCertificates/'
      fn←certpath,name,'-cert.pem'
      ldrc←#.Conga.Init'' ⍝ Need a CONGA instance
      cert←⊃ldrc.ReadCertFromFile fn
      cert.KeyOrigin←{(1⊃⍵)(('-cert'⎕R'-key')2⊃⍵)}cert.CertOrigin
    ∇

    ∇ sa←ServerArgs
      :Access Public Shared
      ⍝ Additional arguments to be provided to Conga.Srv when creating the server
     
      sa←⊂'X509'(ReadTestCert'server/localhost')
      sa,←⊂'SSLValidation' 64 ⍝ 64=request certificate, could be 64+128 to REQUIRE it
    ∇

    ∇ sp←srv ServerProperties name;certpath;_
      :Access Public Shared
      ⍝ Return the Properties to set on the Server instance after it has been created
      ⍝ use the srv ref to access srv and srv.LIB and do it yourself
     
      certpath←(2 ⎕NQ'.' 'GetEnvironment' 'Dyalog'),'/TestCertificates/ca/'
      :If 0≠⊃srv.LIB.SetProp'.' 'RootCertDir'certpath
          ∘∘∘ ⍝ Unable to set certificate directory
      :EndIf
      sp←⍬
    ∇

    ∇ MakeN arg
      :Access Public
      :Implements Constructor :Base arg
     
      Methods←{⍵,⍪1 2∘⊃¨⎕AT¨⍵} 'Primes' 'CountPrimes' ⍝ Callable methods and valences
    ∇

    ∇ onReceive(obj data);cmd;fi;n;⎕TRAP;conn;cert;subject;rc
      :Access Public Override
      
      data←,⊆data 
      
      ⍝ ↓↓↓ Extract and Verify Client ID ↓↓↓ 
      conn←(-(⌽obj)⍳'.')↓obj                    ⍝ Extract connection name
      (rc cert)←srv.LIB.GetProp conn 'PeerCert' ⍝ Should check rc 
      subject←('CN=(.*),' ⎕S '\1')cert.Formatted.Subject
      :If ~subject∊⊂'Geoff Sample'
          _←Respond obj(999('Unauthorised user: ',⊃subject)) ⋄ :Return      
      :EndIf

      :If (⊃⍴Methods)<fi←Methods[;1]⍳⊂cmd←1⊃data ⍝ Command is expected to be a function in this ws
          _←Respond obj(999('Illegal command: ',cmd)) ⋄ :Return
      :EndIf
     
      :If (n←⊃Methods[fi;2])≠¯1+⍴data  ⍝ Number of argument need to match the intance methode
          _←Respond obj(999(cmd,' expects ',(⍕n),'arguments')) ⋄ :Return
      :EndIf
     
      :Select ≢data
      :Case 1 ⍝ Niladic:  fn
          _←Respond obj({0::⎕EN ⎕DM ⋄ 0(⍎⊃⍵)}data)
      :Case 2 ⍝ Mondadic: fn rarg
          _←Respond obj({0::⎕EN ⎕DM ⋄ 0((⍎1⊃⍵)(2⊃⍵))}data)
      :Case 3 ⍝ Dyadic:   fn larg rarg
          _←Respond obj({0::⎕EN ⎕DM ⋄ 0((2⊃⍵)(⍎1⊃⍵)(3⊃⍵))}data)
      :Else
          _←Respond obj(999 'Ill-formed RPC call')
      :EndSelect
    ∇

    :Section Exposed Functions

    ∇ r←Primes n
      :Access Public Shared
      r←⍸2=+⌿0=∘.|⍨⍳n
    ∇
        
    ∇ r←CountPrimes ctl;start;end;m
      :Access Public Shared        
      (start end)←¯2↑1,ctl ⍝ If only one number, start=1
      r←+/2=+⌿m←0=(⍳end)∘.|start+0,⍳end-start 
      r←r,subject
    ∇

    :EndSection Exposed Functions

:EndClass
⍝)(!CountPrimes!!0 0 0 0 0 0 0!0
⍝)(!Primes!!0 0 0 0 0 0 0!0
⍝)(!ServerArgs!!0 0 0 0 0 0 0!0
⍝)(!ServerProperties!!0 0 0 0 0 0 0!0
